﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using System.Collections.ObjectModel;

namespace GraticuleLabelforPSG
{
    public partial class Form1 : Form
    {
        ManagedProj4Projection _proj = null;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                LoadDefaultMap();
                LoadGraticules();

                PlotRoute();

                wfMap.Refresh();
            }
            catch (Exception err)
            {
                MessageBox.Show(err.Message);
            }
        }

        private void LoadDefaultMap()
        {
            try
            {
                if (wfMap.Overlays.Count > 0)
                    wfMap.Overlays.Clear();
                if (wfMap.AdornmentOverlay.Layers.Count > 0)
                    wfMap.AdornmentOverlay.Layers.Clear();

                _proj = new ManagedProj4Projection();
                _proj.InternalProjectionParametersString = ManagedProj4Projection.GetEpsgParametersString(4326);
                _proj.ExternalProjectionParametersString = "+proj=stere +lat_0=90 +lon_0=0 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=m no_defs";

                if (!_proj.IsOpen)
                    _proj.Open();

                wfMap.MapUnit = _proj.GetExternalGeographyUnit();
                wfMap.ZoomLevelSnapping = ZoomLevelSnappingMode.None;
                wfMap.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);

                //GraticuleAdornmentLayer graticuleAdornmentLayer = new GraticuleAdornmentLayer();
                //graticuleAdornmentLayer.Projection = _proj;
                //graticuleAdornmentLayer.GraticuleLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.SimpleColors.Silver, 0.001F, LineDashStyle.Dash, false);
                //wfMap.AdornmentOverlay.Layers.Add("GraticuleAdornmentLayer", graticuleAdornmentLayer);

                ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(@"../../Data/Countries02.shp");
                worldLayer.FeatureSource.Projection = _proj;
                worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
                worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                LayerOverlay overlay_WorldLayer = new LayerOverlay();
                overlay_WorldLayer.Layers.Add("WorldLayer", worldLayer);
                wfMap.Overlays.Add("WorldOverlay", overlay_WorldLayer);

                worldLayer.Open();
                wfMap.CurrentExtent = worldLayer.GetBoundingBox();
                worldLayer.Close();
            }
            catch (Exception err)
            {
                MessageBox.Show(err.Message);
            }
        }

        private void LoadGraticules()
        {
            ShapeFileFeatureLayer shplatlon1 = new ShapeFileFeatureLayer(@"../../Data/Graticule/graticules_1.shp");
            shplatlon1.FeatureSource.Projection = _proj;
            shplatlon1.RequireIndex = false;

            shplatlon1.FeatureSource.Open();
            Collection<Feature> fts1 = shplatlon1.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);
            shplatlon1.FeatureSource.Close();

            shplatlon1.ZoomLevelSet.ZoomLevel10.DefaultLineStyle = new LineStyle(new GeoPen(GeoColor.StandardColors.SlateGray, 0.001F));
            shplatlon1.ZoomLevelSet.ZoomLevel10.DefaultTextStyle = new TextStyle("display", new GeoFont("Arial", 20), new GeoSolidBrush(GeoColors.Red));
            shplatlon1.ZoomLevelSet.ZoomLevel10.DefaultTextStyle.TextLineSegmentRatio = 20;
            shplatlon1.ZoomLevelSet.ZoomLevel10.DefaultTextStyle.SuppressPartialLabels = true;
            shplatlon1.ZoomLevelSet.ZoomLevel10.DefaultTextStyle.GridSize = 5;
            shplatlon1.ZoomLevelSet.ZoomLevel10.DefaultTextStyle.DuplicateRule = LabelDuplicateRule.UnlimitedDuplicateLabels;
            shplatlon1.ZoomLevelSet.ZoomLevel10.DefaultTextStyle.OverlappingRule = LabelOverlappingRule.AllowOverlapping;
            shplatlon1.ZoomLevelSet.ZoomLevel10.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            ShapeFileFeatureLayer shplatlon5 = new ShapeFileFeatureLayer(@"../../Data/Graticule/graticules_5.shp");
            shplatlon5.FeatureSource.Projection = _proj;
            shplatlon5.RequireIndex = false;

            shplatlon5.FeatureSource.Open();
            Collection<Feature> fts5 = shplatlon5.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);
            shplatlon5.FeatureSource.Close();

            shplatlon5.ZoomLevelSet.ZoomLevel07.DefaultLineStyle = new LineStyle(new GeoPen(GeoColor.StandardColors.SlateGray, 0.001F));
            shplatlon5.ZoomLevelSet.ZoomLevel07.DefaultTextStyle = new TextStyle("display", new GeoFont("Arial", 20), new GeoSolidBrush(GeoColors.Red));
            shplatlon5.ZoomLevelSet.ZoomLevel07.DefaultTextStyle.TextLineSegmentRatio = 2;
            shplatlon5.ZoomLevelSet.ZoomLevel07.DefaultTextStyle.SuppressPartialLabels = true;
            shplatlon5.ZoomLevelSet.ZoomLevel07.DefaultTextStyle.GridSize = 5;
            shplatlon5.ZoomLevelSet.ZoomLevel07.DefaultTextStyle.DuplicateRule = LabelDuplicateRule.UnlimitedDuplicateLabels;
            shplatlon5.ZoomLevelSet.ZoomLevel07.DefaultTextStyle.OverlappingRule = LabelOverlappingRule.AllowOverlapping;
            shplatlon5.ZoomLevelSet.ZoomLevel07.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level10;

            ShapeFileFeatureLayer shplatlon10 = new ShapeFileFeatureLayer(@"../../Data/Graticule/graticules_10.shp");
            shplatlon10.FeatureSource.Projection = _proj;
            shplatlon10.RequireIndex = false;

            shplatlon10.FeatureSource.Open();
            Collection<Feature> fts10 = shplatlon10.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);
            shplatlon10.FeatureSource.Close();

            shplatlon10.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = new LineStyle(new GeoPen(GeoColor.StandardColors.SlateGray, 0.001F));
            shplatlon10.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = new TextStyle("display", new GeoFont("Arial", 20), new GeoSolidBrush(GeoColors.Red));
            shplatlon10.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.TextLineSegmentRatio = 2;
            shplatlon10.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.SuppressPartialLabels = true;
            shplatlon10.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.GridSize = 5;
            shplatlon10.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.DuplicateRule = LabelDuplicateRule.UnlimitedDuplicateLabels;
            shplatlon10.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.OverlappingRule = LabelOverlappingRule.AllowOverlapping;
            shplatlon10.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level07;

            ShapeFileFeatureLayer shapeBoundary = new ShapeFileFeatureLayer(@"../../Data/Graticule/graticules_boundary.shp");
            shapeBoundary.FeatureSource.Projection = _proj;
            shapeBoundary.RequireIndex = false;
            shapeBoundary.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = new LineStyle(new GeoPen(GeoColor.StandardColors.SlateGray, 0.001F));
            shapeBoundary.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level07;

            LayerOverlay latlonOverlay = new LayerOverlay();
            latlonOverlay.Layers.Add("Graticule1", shplatlon1);
            latlonOverlay.Layers.Add("Graticule5", shplatlon5);
            latlonOverlay.Layers.Add("Graticule10", shplatlon10);
            latlonOverlay.Layers.Add("GraticuleBoundary", shapeBoundary);
            wfMap.Overlays.Add("GraticuleOverlay", latlonOverlay);
        }

        private void PlotRoute()
        {
            try
            {
                if (wfMap.FindFeatureLayer("RouteLayer") != null)
                    wfMap.Overlays.Remove("RouteOverlay");

                InMemoryFeatureLayer imRouteLayer = new InMemoryFeatureLayer();
                //imRouteLayer.FeatureSource.Projection = _proj;
                imRouteLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new PointStyle(PointSymbolType.Triangle, new GeoSolidBrush(GeoColor.FromArgb(255, 255, 235, 0)), new GeoPen(GeoColor.SimpleColors.Black, 1.5F), 10);
                imRouteLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = new LineStyle(new GeoPen(GeoColor.FromArgb(255, 0, 0, 255), 2));
                imRouteLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                LayerOverlay _routeOverLay = new LayerOverlay();
                _routeOverLay.Layers.Add("RouteLayer", imRouteLayer);
                wfMap.Overlays.Add("RouteOverlay", _routeOverLay);

                PointShape ptFrom = new PointShape("POINT(-118 34)");
                PointShape ptTo = new PointShape("POINT(114 22)");
                Feature ftPtFrom = new Feature(ptFrom);
                Feature ftPtTo = new Feature(ptTo);
                imRouteLayer.InternalFeatures.Add(_proj.ConvertToExternalProjection(ftPtFrom));
                imRouteLayer.InternalFeatures.Add(_proj.ConvertToExternalProjection(ftPtTo));

                //LineShape lnRoute = new LineShape("LINESTRING(" + ptFrom.X + " " + ptFrom.Y + "," + ptTo.X + " " + ptTo.Y + ")");
                MultilineShape lnRoute = GetGreatCircle(ptFrom, ptTo);
                Feature ftLn = new Feature(lnRoute);
                imRouteLayer.InternalFeatures.Add(_proj.ConvertToExternalProjection(ftLn));

                imRouteLayer.Open();
                wfMap.CurrentExtent = imRouteLayer.GetBoundingBox();
                imRouteLayer.Close();
            }
            catch (Exception err)
            {
                MessageBox.Show(err.Message);
            }
        }

        private MultilineShape GetGreatCircle(PointShape pointShape1, PointShape pointShape2)
        {
            MultilineShape greatCircleMultiLineShape = pointShape1.GetShortestLineTo(pointShape2, GeographyUnit.DecimalDegree);

            MultilineShape resultGreatMultiLineShape = new MultilineShape();
            foreach (LineShape lineShape in greatCircleMultiLineShape.Lines)
            {
                if (lineShape.Vertices[0].X < 0)
                {
                    resultGreatMultiLineShape.Lines.Add(lineShape);
                }
                else
                {
                    LineShape resultLineShape = new LineShape();
                    foreach (Vertex vertex in lineShape.Vertices)
                    {
                        resultLineShape.Vertices.Add(vertex);
                    }
                    resultGreatMultiLineShape.Lines.Add(resultLineShape);
                }
            }

            return resultGreatMultiLineShape;
        }

    }
}
